indexing

	description: "Abstraction for child processes that make stdin/stdout %
	%available to their parents."

	author: "Berend de Boer"
	date: "$Date: 2003/01/09 $"
	revision: "$Revision: #3 $"


deferred class

	ABSTRACT_EXEC_PROCESS


inherit

	ABSTRACT_CHILD_PROCESS
		redefine
			wait_for
		end


feature -- Initialization

	make (a_program: STRING; a_arguments: ARRAY[STRING]) is
		require
			program_name_not_empty: a_program /= Void and then not a_program.is_empty
		do
			create program_name.make_from_string (a_program)
			set_arguments (a_arguments)
		end

	make_capture_input (a_program: STRING; a_arguments: ARRAY[STRING]) is
		require
			program_name_not_empty: a_program /= Void and then not a_program.is_empty
		do
			make (a_program, a_arguments)
			set_capture_input (True)
		end

	make_capture_output (a_program: STRING; a_arguments: ARRAY[STRING]) is
		require
			program_name_not_empty: a_program /= Void and then not a_program.is_empty
		do
			make (a_program, a_arguments)
			set_capture_output (True)
		end

	make_capture_io (a_program: STRING; a_arguments: ARRAY[STRING]) is
			-- Why not use threedirectional i/o, because you're getting
			-- yourself in great, great trouble anyway.
			-- A bit of advice: call stdin.close before starting to call
			-- stdout.read_string and such...
		require
			program_name_not_empty: a_program /= Void and then not a_program.is_empty
		do
			make (a_program, a_arguments)
			set_capture_output (True)
			set_capture_input (True)
		end

	make_capture_all (a_program: STRING; a_arguments: ARRAY[STRING]) is
			-- Threedirectional i/o is a great way to get yourself in trouble.
		require
			program_name_not_empty: a_program /= Void and then not a_program.is_empty
		do
			make (a_program, a_arguments)
			set_capture_output (True)
			set_capture_input (True)
			set_capture_error (True)
		end


feature -- (re)set arguments

	set_arguments (a_arguments: ARRAY[STRING]) is
		do
			if a_arguments = Void then
				create arguments.make(0, -1)
			else
				arguments := a_arguments
			end
		end


feature -- i/o capturing

	capture_input: BOOLEAN
			-- is input captured on execute?

	capture_output: BOOLEAN
			-- is output captured on execute?

	capture_error: BOOLEAN
			-- is error captured on execute?

	set_capture_input (on: BOOLEAN) is
		do
			capture_input := on
		end

	set_capture_output (on: BOOLEAN) is
		do
			capture_output := on
		end

	set_capture_error (on: BOOLEAN) is
		do
			capture_error := on
		end


	fd_stdin: ABSTRACT_FILE_DESCRIPTOR

	fd_stdout: ABSTRACT_FILE_DESCRIPTOR

	fd_stderr: ABSTRACT_FILE_DESCRIPTOR


feature -- Execute

	execute is
			-- Executes `program_name'. After execution, at some point in
			-- time, you have to `wait' or `wait_for' for this process to
			-- terminate.
		require
			not_already_started: is_terminated
		deferred
		end


feature -- Actions that parent may execute

	wait_for (suspend: BOOLEAN) is
			-- Wait for child process to terminate if `suspend'.
		deferred
		ensure then
			stdin_closed: is_terminated implies fd_stdin = Void or else fd_stdin.is_closed
			stdout_closed: is_terminated implies fd_stdout = Void or else fd_stdout.is_closed
			stderr_closed: is_terminated implies fd_stderr = Void or else fd_stderr.is_closed
		end


feature -- Accessible state

	program_name: STDC_PATH
			-- program to execute

	arguments: ARRAY[STRING]
			-- arguments to pass to program


invariant

	program_name_not_empty: program_name /= Void and then not program_name.is_empty
	arguments_not_void: arguments /= Void

end
